#include <QApplication>
#include <QGroupBox>
#include <QShortcut>
#include <QStyle>
#include <QVBoxLayout>
#include <QPushButton>

#include <QScreen>

#include <QDebug>
#include <QSysInfo>

#include <algorithm>
#include <iostream>

// 1.2

static QString title()
{
    return QLatin1String(qVersion()) + QLatin1Char(' ') + QGuiApplication::platformName()
        + QLatin1Char(' ') + QApplication::style()->objectName();
}

class Widget : public QWidget
{
public:
    Widget(const QString &t, Qt::WindowFlags f);

};

Widget::Widget(const QString &t, Qt::WindowFlags f) : QWidget (nullptr, f)
{
    auto l = new QVBoxLayout(this);
    auto g = new QGroupBox(t, this);
    l->addWidget(g);
    auto hl = new QHBoxLayout(g);
    auto b = new QPushButton("showMaximized");
    b->setShortcut(Qt::CTRL | Qt::Key_M);
    hl->addWidget(b);
    connect(b, &QPushButton::clicked, this, &QWidget::showMaximized);
    b = new QPushButton("showNormal");
    b->setShortcut(Qt::CTRL | Qt::Key_N);
    hl->addWidget(b);
    connect(b, &QPushButton::clicked, this, &QWidget::showNormal);
    b = new QPushButton("Quit");
    b->setShortcut(Qt::CTRL | Qt::Key_Q);
    hl->addWidget(b);
    connect(b, &QPushButton::clicked, this, &QWidget::close);
    hl->addStretch();
}

int main(int argc, char *argv[])
{
    QStringList args;
    std::copy(argv + 1, argv + argc, std::back_inserter(args));

    QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);

    QApplication app(argc, argv);

    Qt::WindowFlags flags = args.contains("-f")
        ? (Qt::Window | Qt::WindowMinMaxButtonsHint)
        : (Qt::Window | Qt::WindowMinMaxButtonsHint | Qt::FramelessWindowHint);


    QString t = title();
    qDebug().noquote() << t << args
                       << '\n' << flags
                       << "\nWinkey+ArrowUp";

    Widget w(t, flags);
    {
        QDebug d = qDebug().noquote();
        for (auto s : QGuiApplication::screens()) {
            d << (s == w.screen() ? "* " : "  ") << s->name()
              << s->geometry() << "avail=" << s->availableGeometry() << '\n';
        }
        d << '\n';
    }

    w.showNormal();
    int e = QCoreApplication::exec();
    qDebug() << "----------------";
    return e;
}
